cli

package
v0.1.0-alpha.1 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Feb 12, 2026 License: MIT Imports: 14 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func NewIngestCmd

func NewIngestCmd() *cobra.Command

NewIngestCmd creates the ingest command

func SetupGroupedHelp

func SetupGroupedHelp(cmd *cobra.Command)

SetupGroupedHelp configures a command to display flags grouped by category

func SetupIndexHelp

func SetupIndexHelp(cmd *cobra.Command)

SetupIndexHelp configures the search index command to display its flags properly

Types

This section is empty.

Jump to